library(dplyr)
library(tidyr)
library(readxl)
library(readr)
library(tidyverse)
library(cluster)
library(NbClust)
library(factoextra)
library(ggplot2)
library(ggalluvial)
library(factoextra)
library(networkD3)
library(plotly)
library(ggfortify)
library(writexl)
#install.packages("glmnet")
library(glmnet)
#install.packages("coefplot")
library(coefplot)


## establecer directorio de trabajo
setwd("")

## cargar la base de datos con la información de los distintos ejes por municpio
base_otras_var = read.csv("Base_final_clusters.csv")

#### seleccionar las variables a utilizar
base_otras_var <- base_otras_var %>% select(c(ID_MUNICIPIO, DENS_POB, DIST_CAPITAL,CEN_POB_RURAL_DIS,
                                              TRAFICO_DATOS_HAB_2023,TRAFICO_VOZ_HAB_2023,POB_2023, IPM)) %>% rename("CODIGO_DANE_MUNICIPIO"="ID_MUNICIPIO")
## revisar missings
colSums(is.na(base_otras_var))

## Cargar la base de infraestructura del RAN 
infra = readxl::read_xlsx("Base_final_infraestructura.xlsx")

# Calculamos el indicador de penetración para la tecnología IDEN.
infra <- infra %>% mutate(Ind_penetracion_IDEN = TECNOLOGIA_ESTACION_BASE_IDEN - TECNOLOGIA_2021_ESTACION_BASE_IDEN)

# Calculamos el indicador de penetración para la tecnología 2G.
infra <- infra %>% mutate(Ind_penetracion_2G = TECNOLOGIA_ESTACION_BASE_2G - TECNOLOGIA_2021_ESTACION_BASE_2G)

# Calculamos el indicador de penetración para la tecnología 3G.
infra <- infra %>% mutate(Ind_penetracion_3G = TECNOLOGIA_ESTACION_BASE_3G - TECNOLOGIA_2021_ESTACION_BASE_3G)

# Calculamos el indicador de penetración para la tecnología 4G.
infra <- infra %>% mutate(Ind_penetracion_4G = TECNOLOGIA_ESTACION_BASE_4G - TECNOLOGIA_2021_ESTACION_BASE_4G)

# Calculamos el indicador de penetración para la tecnología 5G.
infra <- infra %>% mutate(Ind_penetracion_5G = TECNOLOGIA_ESTACION_BASE_5G - TECNOLOGIA_2021_ESTACION_BASE_5G)

# Eliminamos las columnas que contienen "2021" en su nombre.
infra <- infra %>% select(-contains("2021"))

### revisar missings de la base de datos de infraestructura
colSums(is.na(infra))
## Juntar la base de variables y de infraestructura a nivel de municipio
base_final <- left_join(x = base_otras_var, y=infra, by = "CODIGO_DANE_MUNICIPIO")


### Cargar en el ambiente de trabajo la lista de municipios de la Resolución 6298, se crea con el listado que está en el Anexo 4.8 de la Resolución CRC 5050
munic = readxl::read_xlsx("Municipios Lista Resolución CRC 6298 de 2021.xlsx")
# definir una nueva variable de si el municipio está en la lista de la resolución 6298
munic$munic_res_6298 <- "Está en la lista"

### añadir a la base de datos la variable de si se encuentra en la lista
munic <- munic %>% select(c("munic_res_6298","CODIGO_DANE_MUNICIPIO"))
base_final <- left_join(x = base_final, y=munic, by = "CODIGO_DANE_MUNICIPIO")

## Reemplazar los municipios que no estaban en la lista en la variable construida
base_final$munic_res_6298 <- ifelse(is.na(base_final$munic_res_6298 ), "No está en la lista", base_final$munic_res_6298 )

### remplazar los valores faltantes de tráfico en el RAN por 0 para datos y para voz
base_final$TRAFICO_DATOS_HAB_2023 <- ifelse(is.na(base_final$TRAFICO_DATOS_HAB_2023 ), 0, base_final$TRAFICO_DATOS_HAB_2023 )
base_final$TRAFICO_VOZ_HAB_2023 <- ifelse(is.na(base_final$TRAFICO_VOZ_HAB_2023 ), 0, base_final$TRAFICO_VOZ_HAB_2023 )


### se elimina la variable de Sectores con tecnologia base 1G porque todos tienen 0 en 2023. 
base_final <- base_final %>% select(-c(TECNOLOGIA_ESTACION_BASE_IDEN))

### se elimina la variable de penetracion con tecnologia IDEN
base_final <- base_final %>% select(-c(Ind_penetracion_IDEN))

## quitar la variable de 5g porque solo 1 municipio tuvo este piloto
base_final <- base_final %>% select(-c(TECNOLOGIA_ESTACION_BASE_5G))

## quitar la tasa de penetracion de 5g porque solo 1 municipio tuvo este piloto
base_final <- base_final %>% select(-c(Ind_penetracion_5G))


##########################################################
#### Calcular la capacidad total en red por habitante
##########################################################

#Capacidad por habitante
#Capacidad 2G
base_final <- base_final %>%
  mutate(CAPACIDAD2G = TECNOLOGIA_ESTACION_BASE_2G*3.6*0.05)
#Capacidad 3G
base_final <- base_final %>%
  mutate(CAPACIDAD3G = TECNOLOGIA_ESTACION_BASE_3G*5*0.5)
#Capacidad 4G
base_final <- base_final %>%
  mutate(CAPACIDAD4G = TECNOLOGIA_ESTACION_BASE_4G*15*1.4)
#Capacidad por habitante
base_final <- base_final %>%
  mutate(CAPACIDAD_HAB = (CAPACIDAD2G+CAPACIDAD3G+CAPACIDAD4G)/POB_2023)


### se eliminan missing values 
base_final <- base_final %>% drop_na() 
### revisar que no se hayan eliminado municipios de la lista (74 municipios tienen missing y estaban en la lista)
sum(base_final$munic_res_6298=="Está en la lista")
## Se crea la base de datos con las 9 variables que se van a utilizar.
base_kmedioides <- base_final %>% select(-c(TECNOLOGIA_ESTACION_BASE_2G,TECNOLOGIA_ESTACION_BASE_3G,TECNOLOGIA_ESTACION_BASE_4G,
                                             CAPACIDAD2G,CAPACIDAD3G,CAPACIDAD4G, POB_2023,
                                             Ind_penetracion_2G, Ind_penetracion_3G))


# guardar la base de datos final
write.csv(base_kmedioides, file = "Base_final_ran.csv", row.names = F)


##########################################################
#### Desarrollo######################################
########################################################

#########################################
# Clusterización
#########################################
# Seleccion de variables a trabajar
X <- base_kmedioides %>% select(-c("CODIGO_DANE_MUNICIPIO", "munic_res_6298"))
Y <- base_kmedioides %>% select(c("CODIGO_DANE_MUNICIPIO"))

### se escalan las variables.
X_scale<-scale(X)
#########################################
# Análisis del número óptimo de segmentos
#########################################
# método de explicación del error W
fviz_nbclust(X_scale, pam, method = "wss")

### aplicación de la metodología de Gap Statistic Hastie (2001)
#calcular la estadística de brecha entre la varianza explicada de los datos 
# y de una función uniforme para definir el número de clústeres
gap_stat <- clusGap (X_scale,
                     FUNcluster = pam,
                     K.max = 10, # clústeres máximos a considerar 
                     B = 50) # iteraciones totales de arranque

# trazar el número de conglomerados frente a la estadística computada
fviz_gap_stat(gap_stat)


#########################################
# Algoritmo de segmentación
#########################################
## aplicar los 10 clusters en k-medioides
kmed <- pam(X_scale,k=10)


#######################################################################################
#### Extraer la tabla de estadísticas descriptivas para la interpretación de los cluster
#######################################################################################

### añadir el resultado de la segmentación en la base de datos.
base_kmedioides <- base_kmedioides %>% mutate(kmeans_nuevo = kmed$clustering)
# Creamos una nueva tabla llamada 'tabla_estadisticas3' a partir del marco de datos 'base_kmedioides'.
tabla_estadisticas3 <- base_kmedioides %>%
  # Agrupamos los datos por la columna 'kmeans_nuevo'.
  group_by(kmeans_nuevo) %>%
  # Resumimos los datos dentro de cada grupo.
  summarise(
    num_obs = n(),                               # Número de observaciones en el grupo.
    DENS_POB = mean(DENS_POB),                   # Media de la densidad de población.
    CEN_POB_RURAL_DIS = mean(CEN_POB_RURAL_DIS), # Media del censo de población rural discreta.
    TRAFICO_DATOS_HAB_2023 = mean(TRAFICO_DATOS_HAB_2023), # Media del tráfico de datos por habitante en 2023.
    Ind_penetracion_4G = mean(Ind_penetracion_4G),       # Media del indicador de penetración 4G.
    CAPACIDAD_HAB = mean(CAPACIDAD_HAB),         # Media de la capacidad por habitante.
    TRAFICO_VOZ_HAB_2023 = mean(TRAFICO_VOZ_HAB_2023),   # Media del tráfico de voz por habitante en 2023.
    IPM = mean(IPM),
    distancia_capital= mean(DIST_CAPITAL), # Media del Índice de Pobreza Multidimensional (IPM).
    Num_provedores = mean(Num_provedores)       # Media del número de proveedores.
  ) %>%
  # Reorganizamos los datos en formato largo (long format) utilizando la función 'gather'.
  gather(key = "variable", "value", num_obs, DENS_POB, CEN_POB_RURAL_DIS, TRAFICO_DATOS_HAB_2023, Ind_penetracion_4G, CAPACIDAD_HAB, TRAFICO_VOZ_HAB_2023, IPM, Num_provedores, distancia_capital) %>%
  # Transformamos los datos de nuevo en formato ancho (wide format) utilizando la función 'spread'.
  spread(kmeans_nuevo, value) %>%
  # Agrupamos los datos nuevamente por la columna 'variable'.
  group_by(variable)

# exportar la tabla en formato excel 
writexl::write_xlsx(tabla_estadisticas3, "Estadisticas_idea_3_medioides.xlsx")

############################################
## Diagrama de Sankey
###########################################

# Creamos una tabla de resultados llamada 'resultado3' a partir del marco de datos 'base_kmedioides'.
resultado3 <- base_kmedioides %>%
  # Seleccionamos las columnas 'CODIGO_DANE_MUNICIPIO' y 'munic_res_6298' del marco de datos unicamente para exportar los resultados
  select(CODIGO_DANE_MUNICIPIO, munic_res_6298) %>%
  # Convertimos la columna 'CODIGO_DANE_MUNICIPIO' en tipo de dato character (cadena de caracteres).
  mutate(CODIGO_DANE_MUNICIPIO = as.character(CODIGO_DANE_MUNICIPIO)) %>%
  # Utilizamos 'case_when' para realizar una condición y modificar la columna 'CODIGO_DANE_MUNICIPIO'.
  mutate(CODIGO_DANE_MUNICIPIO = case_when(
    str_length(CODIGO_DANE_MUNICIPIO) == 4 ~ paste0('0', CODIGO_DANE_MUNICIPIO),  # Agregamos un '0' al principio si la longitud es 4.
    TRUE ~ CODIGO_DANE_MUNICIPIO  # Mantenemos el valor original en otros casos.
  )) %>%
  # Agregamos una nueva columna 'kmeans_nuevo' que obtiene los resultados de clustering de 'kmed'.
  mutate(kmeans_nuevo = kmed$clustering)

# Calculamos las frecuencias de interacciones entre categorías para preparar los datos para el diagrama de Sankey.
frequencies3 <- resultado3 %>%
  # Contamos las ocurrencias de combinaciones únicas de 'munic_res_6298' y 'kmeans_nuevo'.
  count(munic_res_6298, kmeans_nuevo) %>%
  # Ordenamos los resultados primero por 'munic_res_6298' y luego de forma descendente por la frecuencia (n).
  arrange(munic_res_6298, desc(n))

# se crea una variable asociada a la lista creada. 
frequencies3 <- frequencies3 %>% mutate(Resolucion_anterior= ifelse(frequencies3$munic_res_6298== "Está en la lista" ,"Lista","No lista"))
frequencies3 <- frequencies3 %>% mutate(clust_3_2_9_8 = ifelse(frequencies3$kmeans_nuevo==3 | frequencies3$kmeans_nuevo==2 | frequencies3$kmeans_nuevo==9| frequencies3$kmeans_nuevo==8,
                                                               "Cluster 10-medioides","Otros"))


# Crear el diagrama de sankey asociado
sankey <- ggplot(data = frequencies3,
                  aes(axis1 =munic_res_6298 ,   # primera variable en el eje x
                      axis2 = kmeans_nuevo, #segunda variable en el eje x
                      y=n)) +
  geom_alluvium(aes(fill = clust_3_2_9_8)) +
  geom_stratum() +
  geom_text(stat = "stratum",
            aes(label = after_stat(stratum))) +
  scale_fill_viridis_d() +
  theme_void() + labs(fill="Municipio")
sankey

# observar la tabla de municipios por cluster
tabla_means <- table(kmed$clustering)
tabla_means


################ LISTA DE MUNICIPIOS K-medioides #############################
###################### Pegue de los nombres del municipio #########################

# juntar todos los municipios 
lista_regulacion <- left_join(base_otras_var, resultado_nombres, by=c("CODIGO_DANE_MUNICIPIO", "MPIO", "DPNOM"))
# reemplazar los municipios sin información por presentes en la lista 
lista_regulacion$Cluster <- ifelse(is.na(lista_regulacion$Cluster), "Lista 10-medioides", lista_regulacion$Cluster)



lista_regulacion <- lista_regulacion  %>% select(c(CODIGO_DANE_MUNICIPIO, MPIO, DPNOM, Cluster))

setwd("G:/Unidades compartidas/CRC- Indice/CRC/Clustering 2/output/tables")
write.csv(lista_regulacion, "lista_RAN.csv")

